import {Apis} from 'zosjs-ws' 
import BigNumber from 'bignumber.js'
import {ChainStore} from 'zosjs/es'

const upColor1 = '#13b884'
const downColor1 = '#eb7676'

class ZosChart {
  constructor () {
    this.firstSymbol = undefined
    this.secondSymbol = undefined
    this.priceprecision = 5
    this.periods = {
      '1': {
        title: '1Min',
        number: 60
      },
      '5': {
        title: '5Min',
        number: 300
      },
      '15': {
        title: '15Min',
        number: 900
      },
      '30': {
        title: '30Min',
        number: 1800
      },
      '60': {
        title: '1H',
        number: 3600
      },
      '240': {
        title: '4H',
        number: 14400
      },
      '1D': {
        title: '1day',
        number: 86400
      }
    }
    this.periodL = []
    this.overrides = {
      // 'paneProperties.background': '#00081A',
      'mainSeriesProperties.showCountdown': false,
      volumePaneSize: 'medium',
      'paneProperties.legendProperties.showLegend': false,
      'paneProperties.legendProperties.showStudyArguments': true,
      'paneProperties.legendProperties.showStudyTitles': true,
      'paneProperties.legendProperties.showStudyValues': true,
      'paneProperties.legendProperties.showSeriesTitle': true,
      'paneProperties.legendProperties.showSeriesOHLC': true,
      'mainSeriesProperties.candleStyle.upColor': upColor1,
      'mainSeriesProperties.candleStyle.downColor': downColor1,
      'mainSeriesProperties.candleStyle.drawWick': true,
      'mainSeriesProperties.candleStyle.drawBorder': true,
      'mainSeriesProperties.candleStyle.borderUpColor': upColor1,
      'mainSeriesProperties.candleStyle.borderDownColor': downColor1,
      'mainSeriesProperties.candleStyle.wickUpColor': upColor1,
      'mainSeriesProperties.candleStyle.wickDownColor': downColor1,
      'mainSeriesProperties.candleStyle.barColorsOnPrevClose': false,
      'mainSeriesProperties.hollowCandleStyle.upColor': upColor1,
      'mainSeriesProperties.hollowCandleStyle.downColor': downColor1,
      'mainSeriesProperties.hollowCandleStyle.drawWick': true,
      'mainSeriesProperties.hollowCandleStyle.drawBorder': true,
      'mainSeriesProperties.hollowCandleStyle.borderUpColor': upColor1,
      'mainSeriesProperties.hollowCandleStyle.borderDownColor': downColor1,
      'mainSeriesProperties.haStyle.upColor': upColor1,
      'mainSeriesProperties.haStyle.downColor': downColor1,
      'mainSeriesProperties.haStyle.drawWick': true,
      'mainSeriesProperties.haStyle.drawBorder': true,
      'mainSeriesProperties.haStyle.borderUpColor': upColor1,
      'mainSeriesProperties.haStyle.borderDownColor': downColor1,
      'mainSeriesProperties.haStyle.barColorsOnPrevClose': false,
      'mainSeriesProperties.barStyle.upColor': upColor1,
      'mainSeriesProperties.barStyle.downColor': downColor1,
      'mainSeriesProperties.barStyle.barColorsOnPrevClose': false,
      'mainSeriesProperties.barStyle.dontDrawOpen': false,
      'mainSeriesProperties.lineStyle.linewidth': 1,
      'mainSeriesProperties.lineStyle.priceSource': 'close',
      'mainSeriesProperties.areaStyle.linewidth': 1,
      'mainSeriesProperties.areaStyle.priceSource': 'close',
      'mainSeriesProperties.style': 1
      // 'mainSeriesProperties.candleStyle.borderColor': 'rgba(52,61,88,0.2)',
      // 'mainSeriesProperties.hollowCandleStyle.borderColor': 'rgba(52,61,88,0.2)',
      // 'mainSeriesProperties.haStyle.borderColor': 'rgba(52,61,88,0.2)',
      // 'mainSeriesProperties.haStyle.wickColor': 'rgba(52,61,88,0.2)',
      // 'mainSeriesProperties.lineStyle.color': 'rgba(52,61,88,0.2)',
      // 'mainSeriesProperties.areaStyle.linecolor': 'rgba(52,61,88,0.2)',
      // 'mainSeriesProperties.hollowCandleStyle.wickColor': undefined,
      // 'paneProperties.crossHairProperties.color': 'rgba(52,61,88,1)',
      // 'paneProperties.horzGridProperties.color': 'rgba(52,61,88,0.2)',
      // 'paneProperties.vertGridProperties.color': 'rgba(52,61,88,0.2)',
      // 'scalesProperties.lineColor': '#343D58',
      // 'scalesProperties.textColor': '#3C434F'
    }
  }
  static getHour (v) {
    let time1 = Number(v)
    time1 = time1 / (3600 * 1000)
    time1 = time1.toFixed(0)
    time1 = time1 * (3600 * 1000)
    return time1
  }
  static getTimeList (days) {
    let timeList = []
    let dynParamObject = ChainStore.getObject('2.0.0', true, true ,true, false)
    if (!dynParamObject) { throw new Error('null dynParamObject') }
    let maxDays = 7
    if (dynParamObject) {
      maxDays = Number(dynParamObject.get('zosparameters').get('max_market_days'))
    }
    if (maxDays > 7) maxDays = 7
    // let timzone = new Date().getTimezoneOffset()
    let nowtime = new Date().getTime()
    let daySeconds = Number(3600 * 24 * 1000)
    let dynGlobalObject = ChainStore.getObject('2.1.0', true, true ,true, false)
    if (!dynGlobalObject) { throw new Error('null dynGlobalObject') }
    nowtime = ZosChart.getTimezoneDateToUinx(dynGlobalObject.get('time'))
    nowtime = Math.floor(nowtime / (3600 * 1000)) * 3600 * 1000
    for (let day = 1; day <= maxDays; day++) {
      let add = {
        id: day,
        time: nowtime + daySeconds * day,
        timeDate: new Date(nowtime + daySeconds * day),
        symbol: day + ' ' + days
      }
      timeList.push(add)
    }
    timeList.forEach(item => {
      item.time = ZosChart.getHour(item.time)
      item.timeDate = new Date(item.time)
    })
    return timeList
  }
  getBuckets () {
    return Apis.instance().history_api().exec('get_market_history_buckets', []).then((info) => {
      this.periods = {}
      this.periodL = []
      info.forEach(item => {
        if (item >= 86400) {
          let day = (item / 86400).toFixed(0)
          let add = {title: day + 'day', number: item}
          this.periods[day + 'D'] = add
          let adl = {title: day + 'D', number: day + 'D'}
          this.periodL.push(adl)
        } else if (item >= 3600) {
          let hour = (item / 3600).toFixed(0)
          let add = {title: hour + 'H', number: item}
          this.periods[(item / 60).toFixed(0).toString()] = add
          let adl = {title: hour + 'H', number: (item / 60).toFixed(0).toString()}
          this.periodL.push(adl)
        } else {
          let min = (item / 60).toFixed(0)
          let add = {title: min + 'Min', number: item}
          this.periods[min.toString()] = add
          let adl = {title: min + 'M', number: min.toString()}
          this.periodL.push(adl)
        }
      })
      let adl = {title: 'INDICATOR', number: 'INDICATOR'}
      this.periodL.push(adl)
      return {periods: this.periods, periodL: this.periodL}
    })
  }
  static getTimezoneDateToUinx (dateString) {
    if (dateString === undefined || dateString === null || dateString.length <= 0) return 0
    if (dateString[dateString.length - 1] !== 'Z') dateString = dateString + 'Z'
    return new BigNumber(new Date(dateString).getTime()).toNumber()
  }
  getPricePrecision () {
    this.priceprecision = 5
    if (this.firstSymbol.precision > this.secondSymbol.precision) this.priceprecision = this.firstSymbol.precision
    else this.priceprecision = this.secondSymbol.precision
    if (this.priceprecision < 5) this.priceprecision = 5
  }
  getMarketHistoryPrice (v, type) {
    let price = 0
    if (v.key.base !== this.firstSymbol.id) {
      price = (v[type + '_base'] * Math.pow(10, this.firstSymbol.precision)) / (v[type + '_quote'] * Math.pow(10, this.secondSymbol.precision))
    } else {
      price = (v[type + '_quote'] * Math.pow(10, this.firstSymbol.precision)) / (v[type + '_base'] * Math.pow(10, this.secondSymbol.precision))
    }
    price = Number(price.toFixed(this.priceprecision))
    return price
  }
  dataFill (interval, array, blockTime, limit) {
    // console.log('dataFill')
    blockTime -= 1 * interval
    let result = []
    let buflen = limit + 2
    if (array.length !== 0) {
      interval = new BigNumber(interval).toNumber()
      let preTime = array[0].createTime
      array.map(({createTime, openPrice, highPrice, lowPrice, closePrice, volume}, index) => {
        if (result.length >= buflen) return
        // console.log(index)
        if (index === 0) {
          result.push({
            createTime: createTime,
            openPrice,
            highPrice,
            lowPrice,
            closePrice,
            volume
          })
          var firstTime = createTime + interval
          while (firstTime < blockTime + interval) {
            result.unshift({
              createTime: firstTime,
              openPrice: closePrice,
              highPrice: closePrice,
              lowPrice: closePrice,
              closePrice: closePrice,
              volume: 0
            })
            firstTime = firstTime + interval
            if (result.length >= buflen) break
          }          
        } else {
          let currentTime = createTime
          if (preTime - currentTime !== interval) {
            while (preTime > currentTime) {
              preTime = preTime - interval
              if (preTime === currentTime) {
                result.push({
                  createTime: preTime,
                  openPrice,
                  highPrice,
                  lowPrice,
                  closePrice,
                  volume: volume
                })
              } else {
                result.push({
                  createTime: preTime,
                  openPrice: closePrice,
                  highPrice: closePrice,
                  lowPrice: closePrice,
                  closePrice: closePrice,
                  volume: preTime === currentTime ? volume : 0
                })
                if (result.length >= buflen) break
              }
            }
          } else {
            preTime = currentTime
            result.push({
              createTime: createTime,
              openPrice,
              highPrice,
              lowPrice,
              closePrice,
              volume
            })
          }
        }
      })
      // console.log(result.length)
      result = result.length <= buflen ? result : result.splice(0, buflen)
    } else {
      result = [{
        createTime: blockTime - interval,
        openPrice: 0,
        highPrice: 0,
        lowPrice: 0,
        closePrice: 0,
        volume: 0
      }, {
        createTime: blockTime,
        openPrice: 0,
        highPrice: 0,
        lowPrice: 0,
        closePrice: 0,
        volume: 0
      }]
    }
    return result.reverse()
  }
  dataUpdate (interval, array, blockTime) {
    // console.log('dataUpdate')
    interval = new BigNumber(interval).toNumber()
    blockTime -= 1 * interval
    blockTime = ((blockTime / interval).toFixed(0) * interval).toFixed(0)
    let result = []
    if (array.length !== 0) {
      array.map(({createTime, openPrice, highPrice, lowPrice, closePrice, volume}, index) => {
        if (index === 0) {
          if (createTime >= blockTime) {
            result.push({
              createTime: createTime,
              openPrice,
              highPrice,
              lowPrice,
              closePrice,
              volume
            })
          } else {
            result.push({
              createTime: blockTime,
              openPrice: closePrice,
              highPrice: closePrice,
              lowPrice: closePrice,
              closePrice: closePrice,
              volume: 0
            })
          }
        }
      })
    } else {
      result = [{
        createTime: blockTime,
        openPrice: 0,
        highPrice: 0,
        lowPrice: 0,
        closePrice: 0,
        volume: 0
      }]
    }
    return result
  }
  getKlineData (period, limit, refurbish, fromtime, blockTimeS) {
    // console.log('getKlineData', period)
    let blockTime = ZosChart.getTimezoneDateToUinx(blockTimeS)
    if (!this.firstSymbol.id || !this.secondSymbol.id) {
      console.error('error symbol')
      return []
    }
    return Apis.instance().history_api().exec('get_market_klinedata', [this.firstSymbol.id, this.secondSymbol.id, period, limit, fromtime, null]).then((groups) => {
      // console.log(groups.length, limit, refurbish)
      groups = groups.map(item => {
        // console.log(item.key.open, this.global.blockTime)
        return {
          createTime: ZosChart.getTimezoneDateToUinx(item.key.open),
          openPrice: this.getMarketHistoryPrice(item, 'open'),
          highPrice: this.getMarketHistoryPrice(item, 'high'),
          lowPrice: this.getMarketHistoryPrice(item, 'low'),
          closePrice: this.getMarketHistoryPrice(item, 'close'),
          volume: item.key.base !== this.firstSymbol.id ? (item.base_volume / Math.pow(10, this.secondSymbol.precision)) : (item.quote_volume / Math.pow(10, this.secondSymbol.precision))
        }
      })
      if (!refurbish) groups = this.dataFill(period * 1000, groups, blockTime, limit)
      else groups = this.dataUpdate(period * 1000, groups, blockTime)
      // console.log(groups[0].createTime, fromtime)
      return groups
    }).catch((error) => {
      console.log(error)
      // this.commonUtils.showToast(error.toString())
      return []
    })
  }
}
export default ZosChart
